function [a e M_0 i RAAN RAAN_dot w n] = GPS_orbits(almanac, n_SV)

% [a e M i RAAN RAAN_dot w] = GPS_orbits(almanac, n_sat)
% Retrieves GPS SV kepler orbital elements from GPS YUMA almanac file
% 
% Inputs:
%   almanac:    path and file name of GPS YUMA almanac in .txt format
%   n_sat:      number of satellites to retrieve data for, up to 31
%
% Outputs:
%   All outputs are 1 x n_SV-vectors with element i being the orbital 
%   element for GPS SV i
%
%   a:          semimajor axis
%   e:          eccentricity
%   M_0:        mean anomaly at epoch
%   i:          inclination
%   RAAN:       right ascenscion of ascending node
%   RAAN_dot:   rate of change of RAAN
%   w:          argument of perigee
%   n:          mean motion
%
%
%
% Author:    Tale Sundlister
% Date:      June 2012
% ________________________________________________________________
%
% Copyright (C) 2012 Tale Sundlister
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% E-mail: tale.sundlisater@gmail.com

global wgs84_GM;

fid = fopen(almanac);
C = textscan(fid, '%s');

% Sanity check on number of satellites
if(n_SV > 31), n_SV = 31; end

a = zeros(1,n_SV);
e = zeros(1,n_SV);
M_0 = zeros(1,n_SV);
i = zeros(1,n_SV);
RAAN = zeros(1,n_SV);
RAAN_dot = zeros(1,n_SV);
w = zeros(1,n_SV);


for index = 1:n_SV
    e(index) = str2double(C{1}{(index-1)*47+13});
    i(index) = str2double(C{1}{(index-1)*47+20});
    RAAN_dot(index) = str2double(C{1}{(index-1)*47+25});
    a(index) = str2double(C{1}{(index-1)*47+29})^2;
    RAAN(index) = str2double(C{1}{(index-1)*47+34});
    w(index) = str2double(C{1}{(index-1)*47+38});
    M_0(index) = str2double(C{1}{(index-1)*47+41});
end

%Calculate mean motion
n = sqrt(wgs84_GM./a.^3);

fclose(fid);