function H = HorbitC(orbit)

% H = HorbitC(orbit) Calculates the linearized observation
% matrix for the position and velocity measurements for a circular orbit
% 
% Inputs:
%   orbit:  4 x 1 vector of circular Kepler orbit elements. The circular
%           assumption means e = 0 and w := 0, therefore they are not
%           tracked as states and the vector is then [a M i RAAN]'
%
% Outputs:
%   H:          6 x 4 linearized observation matrix for the
%               position and velocity measurements
%
% Author:    Tale Sundlister
% Date:      June 2012
% ________________________________________________________________
%
% Copyright (C) 2012 Tale Sundlister
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% E-mail: tale.sundlisater@gmail.com

% For circular orbit
global wgs84_GM;

a = orbit(1);
M = orbit(2);
i = orbit(3);
RAAN = orbit(4);

% Precalculating sines and cosines for speed
c_M = cos(M);
c_i = cos(i);
c_R = cos(RAAN);

s_M = sin(M);
s_i = sin(i);
s_R = sin(RAAN);

% Precalculating commonly used factors
V = sqrt(wgs84_GM/a);
dV_da = V/(2*a);

x_over_a = c_R*c_M - s_R*s_M*c_i;
y_over_a = s_R*c_M + c_R*s_M*c_i;
z_over_a = s_M*s_i;

xdot_over_V = c_R*s_M + s_R*c_i*c_M;
ydot_over_V = s_R*s_M - c_R*c_i*c_M;
zdot_over_V = c_M*s_i;

H_a = [x_over_a;...
       y_over_a;...
       z_over_a;...
       dV_da*xdot_over_V;...
       dV_da*ydot_over_V;...
      -dV_da*zdot_over_V];

H_M = [-a*(c_R*s_M + s_R*c_M*c_i);...
       -a*(s_R*s_M - c_R*c_M*c_i);...
        a*s_i*c_M;...
       -V*(x_over_a);...
       -V*(s_R*c_M + c_R*c_i*s_M);...
       -V*s_i*s_M];

H_i = [a*s_R*s_M*s_i;...
      -a*c_R*s_M*s_i;...
       a*s_M*c_i;...
       V*s_R*s_i*c_M;...
      -V*c_R*s_i*c_M;...
       V*c_i*c_M];

H_RAAN = [-a*(s_R*c_M + c_R*c_i*s_M);...
           a*(c_R*c_M - s_R*c_i*s_M);...
           0;...
           V*(ydot_over_V);...
          -V*(xdot_over_V);...
           0];
   
H = [H_a H_M H_i H_RAAN];