function [q delta_eps gyro_b orbit pos vel N P_res ...
    chosen_sat]= MEKF_ao(gyro, mag, gps_pos, gps_vel,... 
    gps_sat_pos,gps_delta_phi,new_att, new_posvel,...
    sat_indices, init, time)
%-------------------------------------------------------------
% MEKF_ao - Multiplicative Extended Kalman Filter for attitude
% and orbit estimation.
%
% x = [delta_eps gyro_b orbit_a orbit_M orbit_i orbit_RAAN]
% u = [gyro]
% y = [mag delta_r gps_pos gps_vel]
%
% Input         Description         Unit        Frame
% .................................................
% gyro          gyro                [rad/s]     body
% mag           magnetometer        [nT]        body
% gps_pos       gps position        [m]         ecef
% gps_vel       gps velocity        [m/s]       ecef
% gps_delta_phi	phase difference	[rad]       n.a.
%
% Output        Description         Unit        Frame
% ...........................................................
% q             quaternion          [ ]         body_hat to eci
% delta_eps     delta epsilon       [ ]         body to body_hat
% gyro_b        gyro bias           [rad/s]     body
% pos           position            [m]         eci
% vel           velocity            [m/s]       eci
%
% Author:   Tale Sundlister (based on AP_MEKF.m by Harald 
%           Nkland
%           Nonlinear Observer Design for GNSS and IMU 
%           Integration, 2011)
% Date:     May 2012
% -------------------------------------------------------------
global filter_ts gps_att_ts gps_ts gps_lever_arm;
global wgs84_GM res_accept ADOP_tol;

n = 10;                  
    % Number of states
h = filter_ts;           
    % Sampling interval
global L1_lambda;        
    % GPS L1 signal wavelength
global baselines;        
    % Baselines for gps attitude determination
gps_arm = gps_lever_arm; 
    % GPS lever arm, position of antenna in BODY [m]
nLOS = 4;                               
    % Number of GPS satellites to use for attitude estimation
nBL = size(baselines,2);                
    % Number of baselines available for attitude estimation

% Double differencing matrix A, fractional and integer parts
dd_A_frac = diag([1/sqrt(2) 1/sqrt(6) 1/sqrt(12)]);
dd_A_int = [1 -1  0  0 ;1  1 -2  0; 1  1  1 -3];
dd_A = dd_A_frac*dd_A_int;

% Design matrices:

% State variances
var_delta_eps = [1e-4 1e-4 1e-4];	% Gyro variance
var_gyro_b = [1e-5 1e-5 1e-5];      % Gyro bias variance


var_a = 5;                          
         % Semimajor axis a variance
var_M = 1e-5;                       
         % Mean anomaly M variance
var_i = 1e-5;                       
         % Inclination i variance
var_RAAN = 1e-5;                    
         % Right ascencion of the ascending node RAAN variance

var_orbit = [var_a var_M var_i var_RAAN]; 
         % Orbit parameter variance

process  = [var_delta_eps var_gyro_b var_orbit];
Q = diag(process);                  % Process noise

% Measurement variances
var_mag  = [1e-4 1e-4 1e-4];                
                % Magnetometer variance [rad]
var_delta_r = (0.0004^2).*ones(1,nBL*nLOS);	
                % GPS deltarange variance [m]
var_gps_pos = [5 5 5];                      
                % GPS position variance [m]
var_gps_vel = [1e-1 1e-1 1e-1];             
                % GPS velocity variance [m]

meas     = [var_mag var_delta_r var_gps_pos var_gps_vel];
R = diag(meas);                             % Measurement noise
% -------------------------------------------------------------


persistent q_hat delta_eps_bar gyro_b_bar orbit_bar P_bar...
    N_bar Res resdiag prev_indices init_ADOP last_att_time... 
    last_posvel_time;

if(init == 1)
    % First run, set up initial values, initialize integer 
    % ambiguity resolution and choose GPS SV to use for 
    % attitude estimation
    
    delta_eps_bar = [0 0 0]';       % Initial delta epsilon
    gyro_b_bar = [0 0 0]';          % Initial gyro bias
    
    % Find rotation from ecef to eci
    R_ecef2eci = ecef2eci(time);
    
    % Convert GPS position and velocity to ECI
    pos_bar = R_ecef2eci*gps_pos;
    vel_bar = R_ecef2eci*gps_vel;
    
    % Find magnetic field reference vector from IGRF
    pos_sphere = cart2sphere(pos_bar);	
                % = [radius, elevation, azimuth]
    IGRF_sphere = igrf11syn(2014, pos_sphere(1),...
        pos_sphere(2),pos_sphere(3));
    R_sph2ecef = Rsphere2ecef(pos_sphere(2), pos_sphere(3));
    mag_bar_ecef = R_sph2ecef*IGRF_sphere;
    
    % Normalize magnetic field reference vector for use in 
    % q-method
    mag_bar_ecef = mag_bar_ecef/norm(mag_bar_ecef);
    
    
    % Find LOS vectors for all satellites
    LOS_all = gps_sat_pos - gps_pos*ones(1,size(gps_sat_pos,2));
    LOS_all = colnormalize(LOS_all);
    
    % Select satellites
    [LOS_index init_ADOP] = choose_sat(LOS_all,nLOS);
    LOS_bar = LOS_all(:,LOS_index);
    
    prev_indices = sat_indices(LOS_index);
    
    gps_delta_phi = gps_delta_phi(LOS_index,:);
    
    % Resolve integer ambiguities
    [N_bar Res] = intambres(gps_delta_phi',LOS_bar,...
    [0 0 0 0],init);
    
    N_bar = N_bar';
    
    % Find deltaranges and baselines in ECEF
    gps_delta_r = L1_lambda*(gps_delta_phi + N_bar);
    B_ecef_bar = (LOS_bar*LOS_bar')\LOS_bar*gps_delta_r;
    
    
    

    % Convert GPS position and velocity measurement in 
    % ECI to Kepler
    % orbital elements
    [a_bar e_bar M_bar i_bar RAAN_bar w_bar] =... 
     eci2koeC(pos_bar,vel_bar);
    
    % Assuming circular orbit -> e = 0 and w := 0
    orbit_bar = [a_bar M_bar i_bar RAAN_bar]';

    % Find initial attitude from magnetometer and baseline 
    %measurements using q-method
        
    qm_weighting = [10000 1 1 1];
    qm_weighting = qm_weighting/sum(qm_weighting);
    q_hat = qmethod(qm_weighting,...
        colnormalize([R_ecef2eci*mag_bar_ecef... 
        R_ecef2eci*B_ecef_bar]),...
        colnormalize([mag baselines]));
    
    if(q_hat(1) < 0)
        q_hat = -q_hat;
    end
    
    resdiag = zeros(3,4);
    
    last_att_time = time;
    last_posvel_time = time;
    
    P_bar = diag([1e-7 1e-7 1e-7   1e-8 1e-8 1e-8 ...
        10 1e-3 1e-4 1e-4]);  % Initial error covariance
else
    % Calculate estimated ECI position from orbit parameters
    [pos_bar,vel_bar] = koe2eci(orbit_bar(1),0,orbit_bar(2),...
     orbit_bar(3),orbit_bar(4),0);
end



if(new_att == 1 && init == 0)
    
    
    rechoose = 0;
    temp_ind = [0 0 0 0]';
    reset = zeros(1,nLOS);
    % Check if chosen satellites are no longer visible
    for i = 1:nLOS
        ind = find(sat_indices == prev_indices(i), 1);
        if(isempty(ind))
            rechoose = 1;
            break;
        end
        temp_ind(i) = ind;
    end
    
    rechoose = 1;
    
%     % Check if ADOP has degraded
%     if(~rechoose)
%         LOS_bar = gps_sat_pos(:,temp_ind) -...
%         pos_bar*ones(1,nLOS);
%         LOS_bar = colnormalize(LOS_bar);
%         curr_ADOP = adop(LOS_bar);
%         if(curr_ADOP > init_ADOP + ADOP_tol)
%             rechoose = 1;
%         end
%     end
    
    % If necessary, choose new satellites
    if(rechoose)
        %LOS vectors
        LOS_all = gps_sat_pos -...
        pos_bar*ones(1,size(gps_sat_pos,2));
        LOS_all = colnormalize(LOS_all);
               
        % Select satellites
        [LOS_index init_ADOP] = choose_sat(LOS_all,nLOS);
        
        LOS_bar = LOS_all(:,LOS_index);
        new_indices = sat_indices(LOS_index);
        
        % Check for satellite change

        for i = 1:nLOS
            if(new_indices(i) ~= prev_indices(i))
                reset(i) = 1;
            end
        end
        prev_indices = new_indices;
        gps_delta_phi = gps_delta_phi(LOS_index,:);
    else
        gps_delta_phi = gps_delta_phi(temp_ind,:);
    end
    
    
    % Detect outage
    if(time > last_att_time + gps_att_ts + filter_ts)
        reset = ones(1,nLOS);
    end
    
    % Resolve integer ambiguities
    [N_bar Res] = intambres(gps_delta_phi',LOS_bar,reset,init);
    
    N_bar = N_bar';
    %Find delta_r
    gps_delta_r = L1_lambda*(gps_delta_phi + N_bar);
    last_att_time = time;
end

% Predicted rotation matrices:
delta_q_bar = qbuild(delta_eps_bar);
R_bar_body2eci = Rquat(qmult(delta_q_bar,q_hat));
R_bar_eci2body = R_bar_body2eci';

R_ecef2eci = ecef2eci(time);
R_eci2ecef = R_ecef2eci';

R_bar_ecef2body = R_bar_eci2body*R_ecef2eci;
R_bar_body2ecef = R_bar_ecef2body';

%Magnetic field reference vector:
pos_sphere = cart2sphere(pos_bar);	
            % = [radius, elevation, azimuth]
IGRF_sphere = igrf11syn(2014, pos_sphere(1), pos_sphere(2),...
pos_sphere(3));
R_sph2ecef = Rsphere2ecef(pos_sphere(2), pos_sphere(3));
mag_bar_ecef = R_sph2ecef*IGRF_sphere;
mag_bar_body = R_bar_ecef2body*mag_bar_ecef;

if(new_att == 1)
    %Predicted delta_r
    Delta_r_bar = LOS_bar'*R_bar_body2ecef*baselines;
else
    Delta_r_bar = zeros(4,3);
    gps_delta_r = zeros(4,3);
    LOS_bar = zeros(3,4);
end

%Estimated measurement:
y_bar = [mag_bar_body;
         reshape(Delta_r_bar,nLOS*nBL,1);...
         pos_bar;...
         vel_bar]...
      + [meas_bias_mag(-delta_eps_bar,...
         Rquat(qinv(q_hat))*R_ecef2eci*mag_bar_ecef,P_bar);...
         meas_bias_dr(delta_eps_bar,baselines,LOS_bar,...
         R_eci2ecef*Rquat(q_hat),P_bar);...
        [0 0 0]';...
        [0 0 0]'];

% Real measurement:
y = [mag;...
     reshape(gps_delta_r,nLOS*nBL,1);...
     R_ecef2eci*gps_pos;...
     R_ecef2eci*gps_vel];

% Linearized observation matrix
H1 = [2*Smtrx(R_bar_ecef2body*mag_bar_ecef) zeros(3,7)];
if(new_att == 1)
    H2 = [Hdelta_r(R_bar_body2ecef,baselines,LOS_bar)... 
    zeros(nBL*nLOS,7)];
else
    H2 = zeros(nBL*nLOS,10);
end

if(new_posvel == 1)
    H3 = [zeros(6) HorbitC(orbit_bar)];
else
    H3 = zeros(6,10);
end

H = [H1;H2;H3];

% Kalman gain
K = P_bar'*H'/(H*P_bar*H' + R);

% Dead-reckoning
if(new_att == 0)
    % If there is no new attitude data, do not use 
    % attitude measurements
    K(:,4:4+(nLOS*nBL)-1) = zeros(n,nLOS*nBL);
else
    % If there is new attitude data, check the integrity of the 
    % ambiguity solution before using the measurements
    for i = 1:nLOS
        % Integer ambiguity solution integrity check:
        % If three times the square root of the diagonals of 
        % the estimate error covariance matrix of the integer 
        % ambiguity solution is less than res_accept, then the 
        % solution is accepted and the measurement is used
        resdiag(:,i) = 3*sqrt(diag(Res(:,:,i)));
        if(any(resdiag(:,i) > res_accept) ||... 
                any(~isreal(resdiag(:,i))) ||...
                any(isinf(resdiag(:,i))))
            K(:,4+(i-1)) = 0;
            K(:,4+(i-1)+nLOS) = 0;
            K(:,4+(i-1)+2*nLOS) = 0;
        end
    end
end

if(new_posvel == 0)
    % If there is no new position and velocity data, do not 
    % use the position and velocity measurements
    K(:,4+(nBL*nLOS):end) = 0;
end

% Update error state estimate with measurement
delta_eps_hat = delta_eps_bar + K(1:3,:)*(y - y_bar);
gyro_b_hat = gyro_b_bar + K(4:6,:)*(y - y_bar);
if norm(delta_eps_hat)>1, delta_eps_hat =...
        0.999*delta_eps_hat/norm(delta_eps_hat); end

% Update orbit state estimate with measurement
orbit_hat = orbit_bar + K(7:10,:)*(y - y_bar);

% Sanity checks on output
orbit_hat(2) = mod(orbit_hat(2),2*pi);  
            % Mean anomaly defined from 0 to 2*pi
orbit_hat(3) = mod(orbit_hat(3),pi);    
            % Inclination defined from 0 to pi
orbit_hat(4) = mod(orbit_hat(4),2*pi);  
            % RAAN defined from 0 to 2*pi

% Calculate position and velocity from estimated orbit
[pos_hat,vel_hat] = koe2eci(orbit_hat(1),0,...
    orbit_hat(2),orbit_hat(3),orbit_hat(4),0);

% Output:
dq_hat = qbuild(delta_eps_hat);
q_hat = qmult(q_hat,dq_hat);
q_hat = q_hat/norm(q_hat);
delta_eps = delta_eps_hat;
q = q_hat;
gyro_b = gyro_b_hat;
pos = pos_hat;
vel = vel_hat;
orbit = orbit_hat;
N = N_bar;
P_res = resdiag;
chosen_sat = prev_indices;

% Reset error quaternion
delta_eps_hat = [0 0 0]';

% Compute error covariance for updated estimate:
IKH = eye(n) - K*H;
P = IKH*P_bar*IKH' + K*R*K';


% Project ahead:
delta_eps_dot = 0.5*(gyro - gyro_b_hat) +... 
    prop_bias(delta_eps_hat,gyro,gyro_b_hat,P);
gyro_b_dot = [0 0 0]';
orbit_n_hat = sqrt(wgs84_GM/orbit_hat(1)^3);
orbit_dot = [0 orbit_n_hat 0 0]';

delta_eps_bar = delta_eps_hat + h*delta_eps_dot;
gyro_b_bar = gyro_b_hat + h*gyro_b_dot;
orbit_bar = orbit_hat + h*orbit_dot;
if norm(delta_eps_bar)>1, delta_eps_bar =... 
        0.999*delta_eps_bar/norm(delta_eps_bar); end

PHI1 = [-0.5*Smtrx(gyro - gyro_b_hat) -0.5*eye(3) zeros(3,4)];
PHI2 = zeros(3,10);

dM_da = -(3/2)*(wgs84_GM/orbit_hat(1)^5);
PHI3 = [zeros(4,6) [0 dM_da 0 0]' zeros(4,3)];

PHI = eye(n) + h*[PHI1;PHI2;PHI3];
GAMMA = h*eye(n);
P_bar = PHI*P*PHI' + GAMMA*Q*GAMMA';