clear
format shortG

%Parameter initialization

global t_0; t_0 = 0;               %Initial time since ECEF and ECI alignment [s]

%WGS84 defining parameters
global wgs84_a; wgs84_a  = 6378137;                %Equatorial radius (semi-major axis) [m]
global wgs84_b; wgs84_b = 6356752;                %Polar axis radius (semi-minor axis) [m]
global wgs84_f_inv; wgs84_f_inv = 298.257223563;      %Reciprocal flattening
global wgs84_omega_E; wgs84_omega_E = 7.2921151467e-5;	%Angular rate [rad/s]
global wgs84_GM; wgs84_GM = 3.986005e14;       	%Gravitational constant [m^3/s^2]
global wgs84_e; wgs84_e = 0.0818;                 %Eccentricity
global wgs84_c; wgs84_c = 299792458;              %Speed of light [m/s]

%Orbit parameters initial values
orbit_a = wgs84_a + 600000;	%Semi-major axis
orbit_e = 0;            %Eccentricity
orbit_M_0 = 45*pi/180;          %Mean anomaly at epoch, defined from the ascending node for circular orbit, and from the ECI x-axis for circular equatorial orbit
orbit_i = 98*pi/180;	%Inclination
orbit_RAAN = 300*pi/180;         %Right ascencion of the ascending node, defined as 0 for equatorial orbit
orbit_w = 0;            %Argument of perigee, defined as 0 for circular orbit


% Sanity checks
if(orbit_e <= 0)
    orbit_e = 0;
    orbit_w = 0;
end
if(orbit_i == 0)
    orbit_RAAN = 0;
end

orbit_period = sqrt(4*pi^2*orbit_a^3/wgs84_GM);	%Seconds per orbit
orbits_per_day = 86400/orbit_period;            %Orbit revolutions per day

%RAAN perturbation
RAAN_Moon_dot = -0.00338*2.02e-7*cos(orbit_i)/orbits_per_day;

RAAN_Sun_dot = -0.00154*2.02e-7*cos(orbit_i)/orbits_per_day;

orbit_RAAN_dot = RAAN_Moon_dot + RAAN_Sun_dot; % RAAN_dot

%Satellite parameters

sat_m = 1.6;                %Mass [kg]
sat_dim = [0.10 0.10 0.20]; %Satellite dimensions in BODY [m]

%Inertia matrix, satellite modelled as a cuboid of uniform density
sat_I = diag(1/12*sat_m.*[(sat_dim(2)^2 + sat_dim(3)^2)...
    (sat_dim(1)^2 + sat_dim(3)^2) (sat_dim(1)^2 + sat_dim(2)^2)]);

baseline_length = 1;        %Length of baselines [m]

n_antennas = 4;             %Number of antennas

%Antenna positions in BODY [m]
antennas = zeros(3,4);
antennas(:,1) = [-0.5*sat_dim(1)                -0.5*sat_dim(2)                 -0.5*sat_dim(3)]';
antennas(:,2) = [baseline_length-0.5*sat_dim(1) -0.5*sat_dim(2)                 -0.5*sat_dim(3)]';
antennas(:,3) = [-0.5*sat_dim(1)                 baseline_length-0.5*sat_dim(2) -0.5*sat_dim(3)]';
antennas(:,4) = [-0.5*sat_dim(1)                -0.5*sat_dim(2)                  baseline_length-0.5*sat_dim(3)]';

%Baseline vectors in BODY [m]
global baselines; baselines = zeros(3,3);
baselines(:,1) = antennas(:,2) - antennas(:,1);
baselines(:,2) = antennas(:,3) - antennas(:,1);
baselines(:,3) = antennas(:,4) - antennas(:,1);

%GPS Signal parameters
global L1_freq; L1_freq = 1575420000;           % L1 frequency [Hz]
global L1_lambda; L1_lambda = wgs84_c/L1_freq;  % L1 wavelength [m]

%Import GPS orbit parameters
[gps_orbit_a gps_orbit_e gps_orbit_M_0 gps_orbit_i gps_orbit_RAAN...
    gps_orbit_RAAN_dot gps_orbit_w gps_orbit_n]...
    = GPS_orbits('current_gps.txt', 31);

%Convert orbits to ECI
[r_i v_i] = koe2eci(orbit_a,orbit_e,orbit_M_0,orbit_i,orbit_RAAN,orbit_w);
[GPS_r_i GPS_v_i] = koe2eci(gps_orbit_a, gps_orbit_e, gps_orbit_M_0, gps_orbit_i, gps_orbit_RAAN, gps_orbit_w);

%Calculate w_o
orbit_omega_o_io = sqrt(wgs84_GM/norm(r_i)^3);

%Calculate mean motion
orbit_n = sqrt(wgs84_GM/orbit_a^3);

%Initial attitude state
q_0 = rot2q(eci2orbit(r_i,v_i));
omega_b_ib_0 = [0 -0.01 0]';


% Get IGRF coefficients g and h from xls, save as GHcoefficients.
global gh
if exist('GHcoefficients','file')==2
    load('GHcoefficients')
else
    gh=GetIGRF11_Coefficients(1);
end

% Double differencing matrix A, fractional and integer parts
dd_A_frac = diag([1/sqrt(2) 1/sqrt(6) 1/sqrt(12)]);
dd_A_int = [1 -1  0  0 ;1  1 -2  0; 1  1  1 -3];
dd_A = dd_A_frac*dd_A_int;


% Simulated sensor parameters, not necessarily known for calculations
global filter_ts;
filter_ts = 0.1;                % Sampling time of MEKF

% GPS position and velocity parameters
global gps_ts;
gps_ts = 3;                     % GPS position and velocity sampling time [s]
gps_w_1 = 5*ones(3,1);          % GPS position measurement noise variance [m]
gps_w_2 = 1e-1*ones(3,1);       % GPS velocity measurement noise variance [m/s]

global gps_lever_arm;
gps_lever_arm = antennas(:,1);	% GPS antenna position in BODY

% GPS attitude parameters
global gps_att_ts;
gps_att_phnoise = 0.0004;        % standard deviation of phase noise [wavelengths]
gps_att_ts = 1;                 % sampling time for gps attitude sensor

% Parameters for integer ambiguity resolution and satellite selection
global res_accept ADOP_tol;
res_accept = 0.1;               % Integer ambiguity threshold
ADOP_tol = 0.2;                 % ADOP degradation tolerance

% Gyro parameters
gyro_ts = filter_ts;            % Gyro sampling time [s]
gyro_w_1 = (9e-4)^2*ones(3,1);	% Gyro measurement noise variance [rad/s]
gyro_w_2 = 1e-7*ones(3,1);      % Gyro bias dot variance [rad/s^2]
gyro_w_3 = 1e-8*ones(3,1);      % Gyro scale factor error dot variance [ ]
gyro_w_4 = 1e-8*ones(6,1);      % Gyro misalignment angles dot variance [ ]

% Gyro error model initial values
gyro_bias_init = 1e-2*randn(3,1);   % Gyro bias initial value [rad/s]
gyro_kappa_init = 1e-6*randn(3,1);  % Gyro scale factor error initial value [ ]
gyro_alpha_init = 1e-6*randn(6,1);  % Gyro misalignment angles initial value [ ]

gyro_T_1 = 30*60*eye(3);     % Gyro bias time constant matrix [s]
gyro_T_2 = 30*60*eye(3);     % Gyro scale factor error time constant matrix [s]
gyro_T_3 = 30*60*eye(6);     % gyro misalignment angle time constant matrix [s]

% Magnetometer parameters
mag_ts = 0.1;               % Magnetometer sampling time [s]
mag_w_1 = 1e-4*ones(3,1);	% Magnetometer measurement noise variance [rad]
mag_w_2 = 1e-9*ones(3,1);	% Local magnetic disturbance dot variance [rad/s]

mag_dis_init = 1e-5*randn(3,1);

mag_T_1 = 10*60*eye(3);     % Local magnetic disturbance time constant matrix [s]

% Open simulation
Sat6DOF_eci

% Run simulation
set_param('Sat6DOF_eci', 'SimulationCommand', 'start')