function [indices ADOP] = choose_sat(LOS,nLOS)

% [indices ADOP] = choose_sat(LOS,nLOS)
% Chooses a constellation of GPS SV by finding the constellation with
% minimum ADOP. Assumes an ideal baseline configuration with the property
% B*B' = l*eye(nBL) where l is the baseline length and nBL is the number of
% baselines
% 
% Inputs:
%   LOS:        3 x nSV Matrix of column LOS vectors representing the
%               available GPS SV
%   nLOS:       number of SV to choose. 4 are required for good attitude
%               solution
%
% Outputs:
%   indices:	1 x nLOS vector of GPS SV indices, giving their placement
%               in the LOS input matrix
%   ADOP:       Attitude Dilution Of Precision of the chosen constellation
%
% Author:    Tale Sundlister
% Date:      June 2012
% ________________________________________________________________
%
% Copyright (C) 2012 Tale Sundlister
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% E-mail: tale.sundlisater@gmail.com

% Number of GPS SV
nSV = size(LOS,2);

% Find all possible constellation of nLOS GPS SV
comb = nchoosek(1:nSV,nLOS);

% Set initial ADOP value to inf
ADOP = inf;

% Preallocation
indices = zeros(1,nLOS);

for i = 1:size(comb,1)          % Iterate over all possible constellations
    tLOS = LOS(:,comb(i,:));    % Select current constellation of GPS SV
    tADOP = adop(tLOS);         % Calculate ADOP for current constellation
    if(tADOP < ADOP)            % If current constellation is the best so far...
        ADOP = tADOP;           % Accept it and store ADOP value
        indices = comb(i,:);
    end
end