function [N P] = intambres(delta_phi,LOS,reset,init)

% [N P] = intambres(delta_phi,LOS,reset,init) resolves the 
% integer ambiguity problem that rises when calculating 
% deltaranges from phasse differences and line-of-sight vectors.
%
% The algorithm is based on the Fast Integer Ambiguity 
% Resolution algorithm by Lightsey, Crassidis and Markley in 
% which the set of all possible candidates {n1,n2,n3} is first 
% reduced by a geometric inequality before searching through the 
% remaining candidates.
%
% Inputs:
%   delta_phi:  nBL x nLOS matrix of phase differences
%   LOS:        3 x nLOS matrix of column LOS vectors
%   reset:      1 x nLOS boolean vector. If element i is true, 
%               the integer ambiguity solution for LOS i is 
%               reset
%   init:       initialization boolean, set to true (or 1) for 
%               the first run, false (or 0) otherwise
%
% Outputs:
%   N:          nBL x nLOS integer matrix solving the ambiguity 
%               problem
%   P:          nBL x nBL x nLOS error covariance matrix used 
%
% Author:   Tale Sundlister (Based on Fast Integer Ambiguity 
%           Resolution for GPS Attitude Determination by 
%           Lightsey, Crassidis & Markley, 1999.)
% Date:     May 2012
% ___________________________________________________________
%
% Copyright (C) 2012 Tale Sundlister
%
% This program is free software: you can redistribute it 
% and/or modify it under the terms of the GNU General Public 
% License as published by the Free Software Foundation, 
% either version 3 of the License, or (at your option) any 
% later version.
%
% This program is distributed in the hope that it will be 
% useful, but WITHOUT ANY WARRANTY; without even the 
% implied warranty of MERCHANTABILITY or FITNESS FOR A 
% PARTICULAR PURPOSE.  See the GNU General Public License 
% for more details.
%
% You should have received a copy of the GNU General Public 
% License along with this program.  If not, see 
% <http://www.gnu.org/licenses/>.
%
% E-mail: tale.sundlisater@gmail.com

persistent N_candidates candycount J P_inv prev_index;
persistent delta_phi_prev Gamma B B_inv B_inv_cubed GBG;

global baselines L1_lambda;

% Baselines given in wavelengths
b1 = baselines(:,1)/L1_lambda;
b2 = baselines(:,2)/L1_lambda;
b3 = baselines(:,3)/L1_lambda;

% Precompute for speed
sqnorm_b1 = norm(b1)^2;
sqnorm_b2 = norm(b2)^2;
sqnorm_b3 = norm(b3)^2;
b1b2 = dot(b1,b2);
b1b3 = dot(b1,b3);
b2b3 = dot(b2,b3);
upper = floor(norm(b1));
lower = -floor(norm(b1));

%measurement noise
w_ij = 0.0004/L1_lambda; %[wavelengths] typical phase noise

nLOS = size(LOS,2);
nBL = 3;

if(init)
    % First run, preallocate and initialize, 
    % set reset to all true
    N_candidates = -inf.*ones(3,nLOS,(upper-lower+1)^3);
    J = inf.*ones(nLOS,(upper-lower+1)^3);
    P_inv = inf.*ones(3,3,nLOS,(upper-lower+1)^3);
    candycount = zeros(nLOS,3);
    delta_phi_prev = inf*ones(3, nLOS);
    reset = ones(1,nLOS);
    Gamma = (w_ij^-2).*[b1 b2 b3];
    B = (w_ij^-2).*(b1*b1'+b2*b2'+b3*b3');
    B_inv = inv(B);
    B_inv_cubed = B_inv * B_inv * B_inv;
    GBG = Gamma'*B_inv_cubed*Gamma;
else
    % Subsequent runs, check delta_phi to detect integer 
    % crossings, to keep
    % the solution locked
    for j = 1:nLOS
        for i = 1:nBL
            if(delta_phi(i, j) - delta_phi_prev(i, j) > 0.6) 
                % Integer crossing downwards
                if(N_candidates(i,j,prev_index(j)) > lower)
                % if previous integer is above the lowest 
                % possible, decrement it
                    N_candidates(i,j,prev_index(j)) =...
                        N_candidates(i,j,prev_index(j)) - 1;
                else
                    % else, loop around to highest
                    N_candidates(i,j,prev_index(j)) = upper;
                end
            elseif(delta_phi(i,j) - delta_phi_prev(i,j) < -0.6) 
                % Integer crossing upwards
                if(N_candidates(i,j,prev_index(j)) < upper)
                 % if current integer is below the lowest 
                 % possible, increment it
                    N_candidates(i,j,prev_index(j)) =...
                        N_candidates(i,j,prev_index(j)) + 1;
                else
                    % else, loop around to lowest
                    N_candidates(i,j,prev_index(j)) = lower;
                end
            end
        end
    end
end

% Check each GPS signal seperately
for j = 1:nLOS
    
    % Check if the signal has been reset
    if(reset(j))
        
        % The signal has been reset and we must find a new 
        % ambiguity solution {n1j,n2j,n3j}. First we reduce 
        % the search space by using a geometric inequality 
        % that must be satisfied for a candidate
        % solution {n1j,n2j,n3j} to be accepted.
        
        % Reset ambiguity solution for signal j
        N_candidates(:,j,:) = -inf;
        J(j,:) = inf;
        P_inv(:,:,j,:) = inf;
        candycount(j,:) = 0;
        
        % Preallocate candidate storage 1 for speed
        N_temp1 = -inf.*(ones(2, (upper-lower+1)^2));

        % Search through n1 and n2
        for N1=lower:upper,
            for N2=lower:upper,
                
                % Check geometric inequality to accept 
                % or reject current n1 and n2
                if(sqnorm_b1*sqnorm_b2 > (b1b2^2 +...
                        sqnorm_b2*(delta_phi(1,j) + N1)^2 ...
                        - 2*(delta_phi(1,j)...
                        + N1)*(delta_phi(2,j) + N2)*(b1b2) ...
                        + sqnorm_b1*(delta_phi(2,j) + N2)^2))
                    
                    % n1 and n2 accepted as candidates, 
                    % increment candidate
                    % counter, add n1 and n2 to accepted 
                    % candidates
                    candycount(j,1) = candycount(j,1) + 1;
                    N_temp1(:,candycount(j,1)) = [N1 N2]';
                end
            end
        end
        
        % Preallocate candidate storage 2 for speed
        N_temp2 = -inf.*(ones(3,...
            candycount(j,1)*(upper-lower+1)));
        
        % Iterate through accepted n1,n2 pairs to find matching 
        % n3 candidates
        for iter = 1:candycount(j,1)
            
            N1 = N_temp1(1,iter);
            N2 = N_temp1(2,iter);
            
            if(N1 == -inf)
                % If n1 is -inf we have searched through 
                % all candidates
                break;
            end
            
       % Search for n3 candidates for the current n1 and n2
            for N3=lower:upper,
                % Check geometric inequality to accept or
                % reject current n3
                if(sqnorm_b1*sqnorm_b3 > b1b3^2 +...
                        sqnorm_b3*(delta_phi(1,j)+ N1)^2 ...
                        - 2*(delta_phi(1,j) + N1)*...
                        (delta_phi(3,j)+ N3)*(b1b3) ...
                        + sqnorm_b1*(delta_phi(3,j) + N3)^2)
                    % n3 accepted as candidate, increment 
                    % candidate counter, add n1, n2 and n3 
                    % to accepted candidates
                    candycount(j,2) = candycount(j,2) + 1;
                    N_temp2(:,candycount(j,2)) = [N1 N2 N3]';
                end
            end
        end
        
        % Preallocate candidate storage 3 for speed
        N_temp3 = -inf.*(ones(3, candycount(j,2)));
        
        for iter = 1:candycount(j,2)
            N1 = N_temp2(1,iter);
            N2 = N_temp2(2,iter);
            N3 = N_temp2(3,iter);
            if(N1 == -inf)
                % If n1 is -inf we have searched through 
                % all candidates
                break;
            end
            % Check geometric inequality to accept or reject 
            % current candidate set {n1,n2,n3}
            if(sqnorm_b2*sqnorm_b3 > b2b3^2 +...
                    sqnorm_b3*(delta_phi(2,j) + N2)^2 ...
                    - 2*(delta_phi(2,j) + N2)*(delta_phi(3,j)...
                    - N3)*(b2b3) ...
                    + sqnorm_b2*(delta_phi(3,j) + N3)^2)
                % Candidate set accepted, increment candidate 
                % counter and add set to candidates
                candycount(j,3) = candycount(j,3) + 1;
                N_temp3(:,candycount(j,3)) = [N1 N2 N3]';

                
            end
        end
        % Truncate candidate storage 3 to remove empty elements
        N_temp3 = N_temp3(:,1:candycount(j,3));
        % Move candidates to main candidate storage
        N_candidates(:,j,1:candycount(j,3)) = N_temp3;
        
        % Reset score for all candidates
        J(j,1:candycount(j,3)) = 0;
        
        % Reset estimate error covariances for all candidates
        P_inv(:,:,j,1:candycount(j,3)) = 0;
    end
    
    % Compute loss function scores and estimate 
    % error covariances for all candidates
    for candy = 1:candycount(j,3)
        
        % Calculate sigma_j^2
        sigma_sq = (delta_phi(:,j) + N_candidates...
            (:,j,candy))'*GBG*(delta_phi(:,j) +...
            N_candidates(:,j,candy)) -...
            trace(B_inv)^2;
        
        % Calculate loss function
        J(j,candy) = J(j,candy) + 0.5.*((1/sigma_sq)...
            *(norm((B_inv)*Gamma*(delta_phi(:,j) +...
            N_candidates(:,j,candy)))^2 ...
            - norm(LOS(:,j))^2 + trace(B_inv))^2 +...
            log(sigma_sq));
        
        % Calculate estimate error covariance P
        P_inv(:,:,j,candy) = P_inv(:,:,j,candy) +...
            (4/sigma_sq)*(delta_phi(:,j)+N_candidates...
            (:,j,candy))*(delta_phi(:,j)+...
            N_candidates(:,j,candy))';
    end
end

% Preallocate outputs
N = inf*ones(3,nLOS);
P = inf*ones(3,3,nLOS);

% Select the best ambiguity solution {n1j,n2j,n3j} for 
% each sightline j
for j = 1:nLOS
    
    % Find the candidate that minimizes the loss function
    [score index] = min(J(j,1:candycount(j,3)));
    
    
    
    % Add the chosen candidate to the output
    N(:,j) = N_candidates(:,j,index);
    
    % Add the estimate error covariance matrix to the output
    P(:,:,j) = inv(P_inv(:,:,j,index));
    
    % Store the current measurement and solution
    delta_phi_prev = delta_phi;
    prev_index(j) = index;
end