function [R_eci, V_eci] = koe2eci(a, e, M, i, RAAN, w)

% [R_eci, V_eci] = koe2eci(a, e, M, i, RAAN, w)
% Calculates ECI position and velocity from kepler orbital elements.
% 
% Inputs:
%   All inputs are 1 x n_sat-vectors with element i being the orbital 
%   element for satellite i
%
%   a:          semimajor axis
%   e:          eccentricity
%   M:          mean anomaly
%   i:          inclination
%   RAAN:       right ascenscion of ascending node
%   w:          argument of perigee
%
% Outputs:
%   R_eci:      3 x n_sat matrix of ECI position vectors. Column vector i
%               is the position of satellite i in ECI
%   V_eci:      3 x n_sat matrix of ECI velocity vectors. Column vector i
%               is the velocity of satellite i in ECI
%
% Author:    Tale Sundlister
% Date:      June 2012
% ________________________________________________________________
%
% Copyright (C) 2012 Tale Sundlister
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% E-mail: tale.sundlisater@gmail.com

global wgs84_GM;                        %Gravitational constant [m^3/s^2]

if(any(e > 0)) % At least one elliptic orbit
    nu = m2nu(M,e);                             % True anomaly [rad]
    u = nu + w;                             	% Argument of latitude [rad]
    gamma = atan2(e.*sin(nu),1 + e.*cos(nu));   % Flight-path angle [rad]
    r = a.*(1 - e.^2)./(1 + e.*cos(nu));        % Radius [m]
    V = sqrt(wgs84_GM.*(2./r - 1./a));          % Magnitude of velocity [m/s], from vis viva energy integral
else % Circular orbits
    u = M;                                      % Mean anomaly defined from equator
    gamma = 0;
    r = a;
    V = sqrt(wgs84_GM.*(1./a));
    if(i == 0)      % Zero inclination means RAAN is undefined
       RAAN = 0;    % We define it to coincide with the reference direction (vernal equinox)
    end
end

% Precalculating cosines and sines for speed
c_u = cos(u);
c_i = cos(i);
c_R = cos(RAAN);

s_u = sin(u);
s_i = sin(i);
s_R = sin(RAAN);

% ECI positions
x_e = r.*(c_R.*c_u - s_R.*s_u.*c_i);
y_e = r.*(s_R.*c_u + c_R.*s_u.*c_i);
z_e = r.*s_u.*s_i;

% ECI velocities
xdot_e = V.*((x_e./r).*sin(gamma) - cos(gamma).*(c_R.*s_u + s_R.*c_i.*c_u));
ydot_e = -V.*((y_e./r).*sin(gamma) + cos(gamma).*(s_R.*s_u - c_R.*c_i.*c_u));
zdot_e = V.*((z_e./r).*sin(gamma) + cos(gamma).*c_u.*s_i);

% Outputs
R_eci = [x_e; y_e; z_e];
V_eci = [xdot_e; ydot_e; zdot_e];