function nu = m2nu(M,e)

% nu = m2nu(M,e) Calculates true anomaly nu from mean anomaly M and
% eccentricity e
% 
% Inputs:
%   All inputs and outputs are 1 x n_sat-vectors with element i being
%   the orbital element for satellite i
%
%   e:  eccentricity
%   M:  mean anomaly
%
% Outputs:
%   nu: true anomaly
%
% Author:    Tale Sundlister
% Date:      June 2012
% ________________________________________________________________
%
% Copyright (C) 2012 Tale Sundlister
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% E-mail: tale.sundlisater@gmail.com

n_sat = size(M,2);

if(any(e > 0)) % At least one elliptic orbit
    
    E = zeros(1,n_sat);
    
    for i = 1:n_sat
        if (M(i) > -pi && M(i) < 0) || M(i) > pi
            E(i) = M(i) - e(i);
        else
            E(i) = M(i) + e(i);
        end
    end
    tolerance = 10e-8;
    
    Etemp  = E + (M - E + e.*sin(E))./(1-e.*cos(E));
    Etemp2 = E;
    
    % Calculate E using Newton-Rhapson
    for i = 1:n_sat
        while abs(Etemp(i) - Etemp2(i)) > tolerance
            Etemp(i) = Etemp2(i);
            Etemp2(i) = Etemp(i) + (M(i) - Etemp(i) + e(i).*sin(Etemp(i)))./(1-e(i).*cos(Etemp(i)));
        end
    end
    
    E = Etemp2;
    
    % Calculate true anomaly from eccentric anomaly
    nu = atan2((sin(E).*(1-e.^2).^(1/2)),cos(E) - e);
    
else    % Circular orbit, nu defined equal to M
    nu = M;
end