function b_dr = meas_bias_dr(delta_eps,baselines,LOS,R,P)

% b_dr = meas_bias_dr(delta_eps,baselines,LOS,R,P) Calculates the deltarange
% measurement bias due to 2nd-order elements in the measurement function h
%
% Inputs:
%   delta_eps:  vector part of the error quaternion delta q
%   baselines:  3 x nBL matrix of baseline column vectors
%   LOS:        3 x nLOS matrix of LOS column vectors
%   R:          3 x 3 rotation matrix representing the rotation from body_hat to ecef
%   P:          covariance matrix
%
% Outputs:
%   b_dr:       deltarange measurement bias due to 2nd-order elements in
%               the measurement function h
%
% Author:    Tale Sundlister
% Date:      June 2012
% ________________________________________________________________
%
% Copyright (C) 2012 Tale Sundlister
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% E-mail: tale.sundlisater@gmail.com

% delta q reconstructed from delta epsilon
eta = sqrt(1-delta_eps'*delta_eps);
eps = delta_eps;

nBL = size(baselines,2);
nLOS = size(LOS,2);

b_dr = zeros(nBL*nLOS,1);

% 2nd-order differentials of rotation matrix R(delta q)
dRq_deps11 = [0                                     2*(eps(3)/eta + eps(1)^2*eps(3)*eta) -2*(eps(2)/eta + eps(1)^2*eps(2)*eta);...
             -2*(eps(3)/eta + eps(1)^2*eps(3)*eta) -4                                     2*(3*eps(1)/eta + eps(1)^3*eta);...
              2*(eps(2)/eta + eps(1)^2*eps(2)*eta) -2*(3*eps(1)/eta + eps(1)^3*eta)      -4];

dRq_deps12 = [0                                     2*(1 + eps(1)*eps(2)*eps(3)*eta)     -2*(eps(1)/eta + eps(1)*eps(2)^2*eta);...
              2*(1 - eps(1)*eps(2)*eps(3)*eta)      0                                     2*(eps(2)/eta + eps(1)^2*eps(2)*eta);...
              2*(eps(1)/eta + eps(1)*eps(2)^2*eta) -2*(eps(2)/eta + eps(1)^2*eps(2)*eta)  0];

dRq_deps13 = [0                                     2*(eps(1)/eta + eps(1)*eps(3)*eta)    2*(1 - eps(1)*eps(2)*eps(3)*eta);...
             -2*(eps(1)/eta + eps(1)*eps(3)*eta)    0                                     2*(eps(3)/eta + eps(1)^2*eps(3)*eta);...
             2*(1 + eps(1)*eps(2)*eps(3)*eta)      -2*(eps(3)/eta + eps(1)^2*eps(3)*eta)  0];

dRq_deps22 =[-4                                     2*(eps(3)/eta + eps(2)^2*eps(3)*eta) -2*(3*eps(2)/eta + eps(2)^3*eta);...
             -2*(eps(3)/eta + eps(2)^2*eps(3)*eta)  0                                     2*(eps(1)/eta + eps(1)*eps(2)^2*eta);...
              2*(3*eps(2)/eta + eps(2)^3*eta)      -2*(eps(1)/eta + eps(1)*eps(2)^2*eta) -4];
          
dRq_deps23 = [0                                     2*(eps(2)/eta + eps(2)*eps(3)^2*eta) -2*(eps(3)/eta + eps(2)^2*eps(3)*eta);...
             -2*(eps(2)/eta + eps(2)*eps(3)^2*eta)  0                                     2*(1 - eps(1)*eps(2)*eps(3)*eta);...
              2*(eps(3)/eta + eps(2)^2*eps(3)*eta)  2*(1 + eps(1)*eps(2)*eps(3)*eta)      0];
          
dRq_deps33 =[-4                                     2*(3*eps(3)/eta + eps(3)^3*eta)      -2*(eps(2)/eta + eps(2)*eps(3)^2*eta);...
             -2*(3*eps(3)/eta + eps(3)^3*eta)      -4                                     2*(eps(1)/eta + eps(1)*eps(3)^2*eta);...
              2*(eps(2)/eta + eps(2)*eps(3)^2*eta) -2*(eps(1)/eta + eps(1)*eps(3)^2*eta)  0];
          
for j = 1:nBL
    for i = 1:nLOS
        % Add d^2h/(d delta_eps_i)^2
        b_dr(j+i-1) = b_dr(j+i-1) + LOS(:,i)'*R*dRq_deps11*baselines(:,j)*P(1,1);
        b_dr(j+i-1) = b_dr(j+i-1) + LOS(:,i)'*R*dRq_deps22*baselines(:,j)*P(2,2);
        b_dr(j+i-1) = b_dr(j+i-1) + LOS(:,i)'*R*dRq_deps33*baselines(:,j)*P(3,3);
        
        % Add d^2f/(d eps_i d b_j)
        b_dr(j+i-1) = b_dr(j+i-1) + LOS(:,i)'*R*dRq_deps12*baselines(:,j)*P(1,2);
        b_dr(j+i-1) = b_dr(j+i-1) + LOS(:,i)'*R*dRq_deps13*baselines(:,j)*P(1,3);
        b_dr(j+i-1) = b_dr(j+i-1) + LOS(:,i)'*R*dRq_deps23*baselines(:,j)*P(2,3);
    end
end