function b_m = meas_bias_mag(delta_eps,mag_body,P)

% b_m = meas_bias_mag(delta_eps,mag_body,P) Calculates the magnetometer
% measurement bias due to 2nd-order elements in the measurement function h
%
% Inputs:
%   delta_eps:  vector part of the error quaternion delta q
%   mag_body:   magnetometer measurement in BODY frame
%   P:          covariance matrix
%
% Outputs:
%   b_m:        magnetometer measurement bias due to 2nd-order elements in
%               the measurement function h
%
% Author:    Tale Sundlister
% Date:      June 2012
% ________________________________________________________________
%
% Copyright (C) 2012 Tale Sundlister
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% E-mail: tale.sundlisater@gmail.com

% delta q reconstructed from delta epsilon
eta = sqrt(1-delta_eps'*delta_eps);
eps = delta_eps;

b_m = [0 0 0]';

% 2nd-order differentials of rotation matrix R(delta q)
dRq_deps11 = [0                                     2*(eps(3)/eta + eps(1)^2*eps(3)*eta) -2*(eps(2)/eta + eps(1)^2*eps(2)*eta);...
             -2*(eps(3)/eta + eps(1)^2*eps(3)*eta) -4                                     2*(3*eps(1)/eta + eps(1)^3*eta);...
              2*(eps(2)/eta + eps(1)^2*eps(2)*eta) -2*(3*eps(1)/eta + eps(1)^3*eta)      -4];

dRq_deps12 = [0                                     2*(1 + eps(1)*eps(2)*eps(3)*eta)     -2*(eps(1)/eta + eps(1)*eps(2)^2*eta);...
              2*(1 - eps(1)*eps(2)*eps(3)*eta)      0                                     2*(eps(2)/eta + eps(1)^2*eps(2)*eta);...
              2*(eps(1)/eta + eps(1)*eps(2)^2*eta) -2*(eps(2)/eta + eps(1)^2*eps(2)*eta)  0];

dRq_deps13 = [0                                     2*(eps(1)/eta + eps(1)*eps(3)*eta)    2*(1 - eps(1)*eps(2)*eps(3)*eta);...
             -2*(eps(1)/eta + eps(1)*eps(3)*eta)    0                                     2*(eps(3)/eta + eps(1)^2*eps(3)*eta);...
             2*(1 + eps(1)*eps(2)*eps(3)*eta)      -2*(eps(3)/eta + eps(1)^2*eps(3)*eta)  0];

dRq_deps22 =[-4                                     2*(eps(3)/eta + eps(2)^2*eps(3)*eta) -2*(3*eps(2)/eta + eps(2)^3*eta);...
             -2*(eps(3)/eta + eps(2)^2*eps(3)*eta)  0                                     2*(eps(1)/eta + eps(1)*eps(2)^2*eta);...
              2*(3*eps(2)/eta + eps(2)^3*eta)      -2*(eps(1)/eta + eps(1)*eps(2)^2*eta) -4];
          
dRq_deps23 = [0                                     2*(eps(2)/eta + eps(2)*eps(3)^2*eta) -2*(eps(3)/eta + eps(2)^2*eps(3)*eta);...
             -2*(eps(2)/eta + eps(2)*eps(3)^2*eta)  0                                     2*(1 - eps(1)*eps(2)*eps(3)*eta);...
              2*(eps(3)/eta + eps(2)^2*eps(3)*eta)  2*(1 + eps(1)*eps(2)*eps(3)*eta)      0];
          
dRq_deps33 =[-4                                     2*(3*eps(3)/eta + eps(3)^3*eta)      -2*(eps(2)/eta + eps(2)*eps(3)^2*eta);...
             -2*(3*eps(3)/eta + eps(3)^3*eta)      -4                                     2*(eps(1)/eta + eps(1)*eps(3)^2*eta);...
              2*(eps(2)/eta + eps(2)*eps(3)^2*eta) -2*(eps(1)/eta + eps(1)*eps(3)^2*eta)  0];
          

% Add d^2h/(d delta_eps_i)^2
b_m = b_m + dRq_deps11*mag_body*P(1,1);
b_m = b_m + dRq_deps22*mag_body*P(2,2);
b_m = b_m + dRq_deps33*mag_body*P(2,2);

% Add d^2f/(d eps_i d b_j)
b_m = b_m + dRq_deps12*mag_body*P(1,2);
b_m = b_m + dRq_deps13*mag_body*P(1,3);
b_m = b_m + dRq_deps23*mag_body*P(2,3);