function q = qmethod(a, r, b)
% ------------------------------------------------------------------------
% The q-method computes the unit quaternion q = [eta eps1 eps2 eps3]'
% from a set of vector measurements b (3xn) in the body frame. The
% set r (3xn) is the corresponding reference vectors in the reference
% frame, and a (1xn) are weights. It is the solution to an optimization
% problem of minimizing the objective function:
%
% f(q) = sum a_i*norm(r_i-R(q)*b_i)
%
% Input:
% a - weight
% r - reference vector   (NED)
% b - observation vector (BODY)
%
% Author:   Harald Nkland
% Date:     June 2011
% Revised by Jon Anta B Hansen June 2012 - Changed to n vectors, added
% sanity tests
% ------------------------------------------------------------------------

if(size(a,2) ~= size(r,2) || size(a,2) ~= size(b,2))
    error('Number of vectors and weights must be equal')
end

if(size(r,1) ~= 3 || size(b,1) ~= 3)
    error('Vectors must be of dimension 3x1');
end

n = size(r,2);

Z = zeros(3,1);
B = zeros(3);

for i = 1:n
    r(:,i) = r(:,i)/norm(r(:,i));
    b(:,i) = b(:,i)/norm(r(:,i));
    Z = Z + a(i)*Smtrx(r(:,i))*b(:,i);
    B = B + a(i)*r(:,i)*b(:,i)';
end
S = B + B';
sigma = trace(B);

% Compute the K matrix
K = [ -sigma     Z'
    Z        -S+sigma*eye(3) ];

% Find the eigenvector for the smallest eigenvalue of K
[V,E] = eig(K);
[m,i] = min([E(1,1) E(2,2) E(3,3) E(4,4)]);
q = V(:,i);
q = q/norm(q);
