close all;

global wgs84_a;

% Change this to zero to disable runtime visualization. Plots while running
% significantly increase the runtime, but are informative and look cool
visualization = 0;

% Retrieve measurements and actual values from structures into more
% convenient arrays
t = time;                                   % Time vector
steps = size(t,1);                          % Number of steps

omega_measured = omega_imu.signals.values';	% Gyro measurement
mag_measured = m_imu.signals.values';       % Magnetometer measurement
pos_gps = r_gps.signals.values';            % GPS Position measurement
vel_gps = v_gps.signals.values';            % GPS Velocity measurement
gpssv_pos = r_gpssat.signals.values;        % GPS SV Positions
delta_phi_all = delta_phi.signals.values;   % Phase difference measurement
indices = sat_indices.signals.values;       % GPS SV PRN Vector
vis =r_gpssat.signals.valueDimensions(:,2); % Number of visible GPS SV

att_valid = gps_att_valid.signals.values;   % Delta phi measurement validity
pv_valid = gps_pos_vel_valid.signals.values;% Position and velocity measurement validity

orb_act = orbit_actual.signals.values';     % Actual orbit parameters
N_act_all = N_actual.signals.values;        % Actual deltarange integers
q_i2b_act = q_i2b_actual.signals.values';   % Actual attitude
g_b_act = gyro_bias_actual.signals.values'; % Actual gyro bias
pos_act = r_actual.signals.values';         % Actual position
vel_act = v_actual.signals.values';         % Actual velocity
omega_act=omega_bib_actual.signals.values'; % Actual angular velocity


% Preallocating matrices for speed
delta_phi_measured = zeros(4,3,steps);
N_est = zeros(4,3,steps);
N_act = zeros(4,3,steps);
q_b2i_est = zeros(4,steps);
q_i2b_est = zeros(4,steps);
q_error = zeros(4,steps);
orb_est = zeros(size(orb_act));
pos_act_eci = zeros(size(pos_act));
vel_act_eci = zeros(size(vel_act));
pos_gps_eci = zeros(size(pos_act));
vel_gps_eci = zeros(size(vel_act));
gyro_b_est = zeros(3,steps);
gyro_b_error = zeros(3,steps);
delta_eps = zeros(3,steps);
pos_est = zeros(3,steps);
pos_error = zeros(3,steps);
vel_est = zeros(3,steps);
vel_error = zeros(3,steps);
P_res = zeros(3,4,steps);
chosen_sat = zeros(4,steps);

[X Y Z] = sphere;
X = wgs84_a*X;
Y = wgs84_a*Y;
Z = wgs84_a*Z;


s_freq = floor(1/filter_ts);

% Set GPS outage: first number is start of outage in seconds, second number
% is end of outage in seconds
att_valid(floor(s_freq*101):floor(s_freq*150)) = 0;
pv_valid(floor(s_freq*101):floor(s_freq*150)) = 0;

for i = 1:steps
    
    % Check if first run
    if i == 1
        init = 1;
    else
        init = 0;
    end
    
    % Run filter
    [q_b2i_est(:,i) delta_eps(:,i) gyro_b_est(:,i) orb_est([1 3 4 5],i)...
        pos_est(:,i) vel_est(:,i) N_est(:,:,i) P_res(:,:,i) chosen_sat(:,i)]...
        = MEKF_ao(omega_measured(:,i), mag_measured(:,i),...
        pos_gps(:,i), vel_gps(:,i), gpssv_pos(:,1:vis(i),i),...
        delta_phi_all(1:vis(i),:,i),att_valid(i),pv_valid(i),...
        indices(:,1:vis(i),i), init, t(i));
    
    % Find the chosen satellite indices from the identifiers
    chosen_indices = zeros(size(chosen_sat(:,i)));
    for j = 1:size(chosen_sat(:,i),1)
        chosen_indices(j) = find(chosen_sat(j,i)' == indices(:,1:vis(i),i));
    end
    % Select chosen actual measurements
    delta_phi_measured(:,:,i) = delta_phi_all(chosen_indices,:,i);
    N_act(:,:,i) = N_act_all(chosen_indices,:,i);
    
    % Crop estimate error covariance for plotting, we are interested in the
    % region between 0 and 1
    for a = 1:3
        for b = 1:4
            if(P_res(a,b,i) > 1 || ~isreal(P_res(a,b,i)))
                P_res(a,b,i) = 1;
            end
        end
    end
    
    % Invert the estimated quaternion for plotting
    q_i2b_est(:,i) = qinv(q_b2i_est(:,i));
    
    % Set estimated and actual quaternion to have the same sign in eta
    if(q_i2b_est(1,i) > 0 && q_i2b_act(1,i) < 0 || q_i2b_est(1,i) < 0 && q_i2b_act(1,i) > 0)
        q_i2b_est(:,i) = -q_i2b_est(:,i);
    end
    
    % Compute error quaternion
    q_error(:,i) = qmult(q_b2i_est(:,i),q_i2b_act(:,i));
    
    % Rotate all positions and velocities to ECI
    R_ecef2eci = ecef2eci(t(i));
    R_eci2ecef = R_ecef2eci';
    pos_act_eci(:,i) = R_ecef2eci*pos_act(:,i);
    vel_act_eci(:,i) = R_ecef2eci*vel_act(:,i);
    
    pos_gps_eci(:,i) = R_ecef2eci*pos_gps(:,i);
    vel_gps_eci(:,i) = R_ecef2eci*vel_gps(:,i);
    
    if(visualization)
        figure(1)
        subplot(1,2,1)
        
        surf(X,Y,Z)
        hold on
        colormap([1 1 1])
        axis auto
        cs_1 = 600000*eye(3);
        
        % cs_orb = Rquat(q_i2o_act(:,1))*cs;
        cs_orb = eci2orbit(pos_act_eci(:,i),vel_act_eci(:,i))'*cs_1;
        cs_body = Rquat(q_i2b_act(:,i))*0.9*cs_1;
        cs_body_est = Rquat(q_i2b_est(:,i))*0.75*cs_1;
        
        cs_eci = (wgs84_a + 1000000)*eye(3);
        cs_ecef = R_eci2ecef*cs_eci;
        
        
        % Plot actual position vector
        plot3([0 pos_act_eci(1,i)]', [0 pos_act_eci(2,i)]', [0 pos_act_eci(3,i)]','-s')
        
        
        % Plot estimated position vector
        plot3([0 pos_est(1,i)]', [0 pos_est(2,i)]', [0 pos_est(3,i)]','--s')
        
        % Plot velocity vector
        plot3([pos_act_eci(1,i) pos_act_eci(1,i)+100*vel_act_eci(1,i)]',...
            [pos_act_eci(2,i) pos_act_eci(2,i)+100*vel_act_eci(2,i)]',...
            [pos_act_eci(3,i) pos_act_eci(3,i)+100*vel_act_eci(3,i)]','-+')
        
        % Plot estimated velocity vector
        plot3([pos_est(1,i) pos_est(1,i)+100*vel_est(1,i)]',...
            [pos_est(2,i) pos_est(2,i)+100*vel_est(2,i)]',...
            [pos_est(3,i) pos_est(3,i)+100*vel_est(3,i)]','--+m')
        
        % Plot ECEF coordinate frame
        plot3([0 cs_ecef(1,1)]',[0 cs_ecef(2,1)]',[0 cs_ecef(2,1)]','--xr')
        plot3([0 cs_ecef(1,2)]',[0 cs_ecef(2,2)]',[0 cs_ecef(3,2)]','--dr')
        plot3([0 cs_ecef(1,3)]',[0 cs_ecef(2,3)]',[0 cs_ecef(3,3)]','--r')
        
        % Plot ECI coordinate frame
        plot3([0 cs_eci(1,1)]',[0 cs_eci(2,1)]',[0 cs_eci(2,1)]','--xk')
        plot3([0 cs_eci(1,2)]',[0 cs_eci(2,2)]',[0 cs_eci(3,2)]','--dk')
        plot3([0 cs_eci(1,3)]',[0 cs_eci(2,3)]',[0 cs_eci(3,3)]','--k')
        
        % Plot ORBIT coordinate frame
        plot3([pos_act_eci(1,i) (pos_act_eci(1,i)+cs_orb(1,1))]',...
            [pos_act_eci(2,i) pos_act_eci(2,i) + cs_orb(2,1)]',...
            [pos_act_eci(3,i) pos_act_eci(3,i) + cs_orb(3,1)]','-xr')
        plot3([pos_act_eci(1,i) (pos_act_eci(1,i)+cs_orb(1,2))]',...
            [pos_act_eci(2,i) pos_act_eci(2,i) + cs_orb(2,2)]',...
            [pos_act_eci(3,i) pos_act_eci(3,i) + cs_orb(3,2)]','-dr')
        plot3([pos_act_eci(1,i) (pos_act_eci(1,i)+cs_orb(1,3))]',...
            [pos_act_eci(2,i) pos_act_eci(2,i) + cs_orb(2,3)]',...
            [pos_act_eci(3,i) pos_act_eci(3,i) + cs_orb(3,3)]','r')
        
        % Plot actual BODY frame
        plot3([pos_act_eci(1,i) (pos_act_eci(1,i)+cs_body(1,1))]',...
            [pos_act_eci(2,i) pos_act_eci(2,i) + cs_body(2,1)]',...
            [pos_act_eci(3,i) pos_act_eci(3,i) + cs_body(3,1)]','-xg')
        plot3([pos_act_eci(1,i) (pos_act_eci(1,i)+cs_body(1,2))]',...
            [pos_act_eci(2,i) pos_act_eci(2,i) + cs_body(2,2)]',...
            [pos_act_eci(3,i) pos_act_eci(3,i) + cs_body(3,2)]','-dg')
        plot3([pos_act_eci(1,i) (pos_act_eci(1,i)+cs_body(1,3))]',...
            [pos_act_eci(2,i) pos_act_eci(2,i) + cs_body(2,3)]',...
            [pos_act_eci(3,i) pos_act_eci(3,i) + cs_body(3,3)]','g')
        
        % Plot estimated BODY frame
        plot3([pos_est(1,i) (pos_est(1,i)+cs_body_est(1,1))]',...
            [pos_est(2,i) pos_est(2,i) + cs_body_est(2,1)]',...
            [pos_est(3,i) pos_est(3,i) + cs_body_est(3,1)]','-xc')
        plot3([pos_est(1,i) (pos_est(1,i)+cs_body_est(1,2))]',...
            [pos_est(2,i) pos_est(2,i) + cs_body_est(2,2)]',...
            [pos_est(3,i) pos_est(3,i) + cs_body_est(3,2)]','-dc')
        plot3([pos_est(1,i) (pos_est(1,i) + cs_body_est(1,3))]',...
            [pos_est(2,i) pos_est(2,i) + cs_body_est(2,3)]',...
            [pos_est(3,i) pos_est(3,i) + cs_body_est(3,3)]','c')
        
        % Plot chosen GPS SV LOS vectors
        for j = 1:4
            plot3([pos_act_eci(1,i) gpssv_pos(1,chosen_indices(j),i)]',...
                [pos_act_eci(2,i) gpssv_pos(2,chosen_indices(j),i)]',...
                [pos_act_eci(3,i) gpssv_pos(3,chosen_indices(j),i)]','-x');
        end
        
        xlabel('x^i')
        ylabel('y^i')
        zlabel('z^i')
        axis equal
        axis([pos_act_eci(1,i)-1000000 pos_act_eci(1,i)+1000000 ...
              pos_act_eci(2,i)-1000000 pos_act_eci(2,i)+1000000 ...
              pos_act_eci(3,i)-1000000 pos_act_eci(3,i)+1000000]);
        
        hold off
        
        % Plot rotation error
        subplot(1,2,2)
        cs = eye(3);
        cs_error = Rquat(q_error(:,i))*cs;
        
        plot3([0 cs(1,1)]',[0 cs(2,1)]',[0 cs(3,1)]','-xk')
        hold on
        plot3([0 cs(1,2)]',[0 cs(2,2)]',[0 cs(3,2)]','-dk')
        plot3([0 cs(1,3)]',[0 cs(2,3)]',[0 cs(3,3)]','k')
        
        plot3([0 cs_error(1,1)]',[0 cs_error(2,1)]',[0 cs_error(3,1)]','-xr')
        plot3([0 cs_error(1,2)]',[0 cs_error(2,2)]',[0 cs_error(3,2)]','-dr')
        plot3([0 cs_error(1,3)]',[0 cs_error(2,3)]',[0 cs_error(3,3)]','r')
        axis equal
        hold off
    end
end

figure(2)

subplot(2,1,1)
plot(t(1:steps),q_i2b_act(:,1:steps),t(1:steps),q_i2b_est(:,1:steps),'--')
xlabel('time [s]')
ylabel('attitude [ ]')
legend('eta actual','epsilon_1 actual','epsilon_2 actual','epsilon_3 actual','eta estimate','epsilon_1 estimate','epsilon_2 estimate','epsilon_3 estimate')

subplot(2,1,2)
plot(t(1:steps),q_error(:,1:steps))
xlabel('time [s]')
ylabel('attitude error quaternion [ ]')
legend('error eta','error epsilon_1','error epsilon_2','error epsilon_3')

figure(3)
subplot(2,1,1)
plot(t(1:steps),omega_act(:,1:steps),t(1:steps),omega_measured(:,1:steps) - gyro_b_est(:,1:steps),'--')
xlabel('time [s]')
ylabel('angular velocity [rad/s]')
legend('omega x actual','omega y actual','omega z actual','omega x estimate','omega y estimate','omega z estimate')

subplot(2,1,2)
plot(t(1:steps),omega_act(:,1:steps) - (omega_measured(:,1:steps) - gyro_b_est(:,1:steps)))
xlabel('time [s]')
ylabel('angular velocity error [rad/s]')
legend('omega_x error','omega_y error','omega_z error')

figure(4)
subplot(2,1,1)
plot(t(1:steps),pos_act_eci(:,1:steps),t(1:steps),pos_est(:,1:steps),'--',t(1:steps),pos_gps_eci(:,1:steps),':')
xlabel('time [s]')
ylabel('position [m]')
legend('x position','y position','z position','estimated x position', 'estimated y position', 'estimated z position','gps x position', 'gps y position', 'gps z position')

subplot(2,1,2)
plot(t(1:steps),pos_act_eci(:,1:steps) - pos_est(:,1:steps))
xlabel('time [s]')
ylabel('position error [m]')
legend('x error', 'y error', 'z error')

% subplot(2,2,3)
% plot(t(1:steps),(ones(3,1)*posvel_valid(1:steps)').*(pos_gps_eci(:,1:steps) - pos_est(:,1:steps)))
% xlabel('time [s]')
% ylabel('position error (GPS vs est) [m]')
% legend('x error','y error','z error')
%
% subplot(2,2,4)
% plot(t(1:steps),(ones(3,1)*posvel_valid(1:steps)').*(pos_act_eci(:,1:steps) - pos_gps_eci(:,1:steps)))
% xlabel('time [s]')
% ylabel('position error (act vs GPS) [m]')
% legend('x error','y error','z error')

figure(5)
subplot(2,1,1)
plot(t(1:steps),vel_act_eci(:,1:steps),t(1:steps),vel_est(:,1:steps),'--')
xlabel('time [s]')
ylabel('velocity [m/s]')
legend('actual velocity','actual velocity','actual velocity','estimated velocity','estimated velocity','estimated velocity')

subplot(2,1,2)
plot(t(1:steps),vel_act_eci(:,1:steps) - vel_est(:,1:steps))
xlabel('time [s]')
ylabel('velocity error [m/s]')
legend('x error','y error','z error')

% subplot(2,2,3)
% plot(t(1:steps),vel_gps_eci(:,1:steps) - vel_est(:,1:steps))
% xlabel('time [s]')
% ylabel('velocity error (GPS vs est) [m/s]')
% legend('x error','y error','z error')
%
% subplot(2,2,4)
% plot(t(1:steps),vel_act_eci(:,1:steps) - vel_gps_eci(:,1:steps))
% xlabel('time [s]')
% ylabel('velocity error (act vs GPS) [m/s]')
% legend('x error','y error','z error')

figure(6)
subplot(2,1,1)
plot(t(1:steps),g_b_act(:,1:steps),t(1:steps),gyro_b_est(:,1:steps),'--')
xlabel('time [s]')
ylabel('gyro bias [rad/s]')
legend('x bias','y bias','z bias','x bias estimate','y bias estimate','z bias estimate')

subplot(2,1,2)
plot(t(1:steps),g_b_act(:,1:steps)-gyro_b_est(:,1:steps))
xlabel('time [s]')
ylabel('gyro bias error [rad/s]')
legend('x bias error','y bias error','z bias error')

figure(8)
subplot(2,1,1)
plot(t(1:steps),orb_act(1,1:steps),t(1:steps),orb_est(1,1:steps),'--')
xlabel('time [s]')
ylabel('semimajor axis a [m]')
legend('actual a','estimated a')

subplot(2,1,2)
plot(t(1:steps),orb_act(1,1:steps) - orb_est(1,1:steps))
xlabel('time [s]')
ylabel('semimajor axis a estimate error [m]')

% figure(9)
% subplot(2,1,1)
% plot(t(1:steps),orb_act(2,1:steps),t(1:steps),orb_est(2,1:steps),'--')
% xlabel('time [s]')
% ylabel('eccentricity e [ ]')
% legend('actual e','estimated e')
%
% subplot(2,1,2)
% plot(t(1:steps),orb_act(2,1:steps) - orb_est(2,1:steps))
% xlabel('time [s]')
% ylabel('eccentricity e estimate error [ ]')

figure(10)
subplot(2,1,1)
plot(t(1:steps),orb_act(3,1:steps),t(1:steps),orb_est(3,1:steps),'--')
xlabel('time [s]')
ylabel('mean anomaly M [ ]')
legend('actual M','estimated M')

subplot(2,1,2)
plot(t(1:steps),orb_act(3,1:steps) - orb_est(3,1:steps))
xlabel('time [s]')
ylabel('mean anomaly M estimate error [ ]')

figure(11)
subplot(2,1,1)
plot(t(1:steps),orb_act(4,1:steps),t(1:steps),orb_est(4,1:steps),'--')
xlabel('time [s]')
ylabel('inclination i [rad]')
legend('actual i','estimated i')

subplot(2,1,2)
plot(t(1:steps),orb_act(4,1:steps) - orb_est(4,1:steps))
xlabel('time [s]')
ylabel('inclination i estimate error [ ]')

figure(12)
subplot(2,1,1)
plot(t(1:steps),orb_act(5,1:steps),t(1:steps),orb_est(5,1:steps),'--')
xlabel('time [s]')
ylabel('Right Ascenscion of the Ascending Node Omega [rad]')
legend('actual Omega','estimated Omega')

subplot(2,1,2)
plot(t(1:steps),orb_act(5,1:steps) - orb_est(5,1:steps))
xlabel('time [s]')
ylabel('RAAN Omega estimate error [ ]')


for i = 1:4
    figure(11 + i)
    % subplot(2,2,1)
    % plot(t(1:steps),reshape(N_act,12,steps),t(1:steps),reshape(N_est,12,steps),'--')
    % xlabel('time [s]')
    % ylabel('integer ambiguity [ ]')
    
    subplot(3,1,1)
    plot(t(1:steps),reshape(N_act(i,:,1:steps),3,steps)-reshape(N_est(i,:,1:steps),3,steps))
    xlabel('time [s]')
    ylabel('integer ambiguity error [ ]')
    legend('baseline 1','baseline 2','baseline 3')
    
    subplot(3,1,2)
    plot(t(1:steps),reshape(delta_phi_measured(i,:,1:steps),3,steps))
    xlabel('time [s]')
    ylabel('delta phi [wavelengths]')
    legend('baseline 1','baseline 2','baseline 3')
    
    subplot(3,1,3)
    plot(t(1:steps),reshape(P_res(:,i,1:steps),3,steps),[0 t(steps)]',[0.1 0.1]','--')
    xlabel('time [s]')
    ylabel('integrity check value [ ]')
    legend('baseline 1','baseline 2','baseline 3','acceptance threshold')
end